import subprocess


def miqsel_cmd(cmd):
    """
    Runs a miqsel shell command, and returns the output, err, returncode
    """
    ret = subprocess.Popen(
        cmd,
        shell=True,
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        close_fds=True,
    )
    out, err = ret.communicate()
    returncode = ret.returncode
    return out.decode(), err.decode(), returncode



import json
import os
import sys
from pathlib import Path

import click


DEFAULT_CONFIG = {
    "engine": "auto",
    "viewer": "auto",
    "data_dir": "default",
    "image": "selenium/standalone-chrome:latest",
    "name": "miqsel_container",
    "network": "default",
    "server_port": 4444,
    "vnc_port": 5999,
    "shm_size": "2g",
}
CONTAINER_ENGINES = ["podman", "docker"]
VNC_VIEWERS = ["vncviewer", "vinagre", "xdg-open"]


class Configuration:
    """read write conf yaml files"""

    def __init__(self, conf_file=None):
        self.conf_file = conf_file

    @property
    def config_path(self):
        xdg_config_home = os.environ.get("XDG_CONFIG_HOME")

        if xdg_config_home:
            config_home = Path(xdg_config_home)
        else:
            config_home = Path.home().joinpath(".config")

        conf_path = config_home.joinpath("miqsel").joinpath("config.json")
        conf_path.parent.mkdir(parents=True, exist_ok=True)
        return conf_path

    def read(self):
        if self.config_path.exists():
            return json.loads(self.config_path.read_text(encoding="utf-8"))
        else:
            self.write(DEFAULT_CONFIG)
            return DEFAULT_CONFIG

    def write(self, cfg: dict):
        self.config_path.write_text(json.dumps(cfg), encoding="utf-8")


@click.command(help="Set Configuration")
@click.option("--reset", is_flag=True, help="Reset configuration to default.")
def config(reset):
    """Config command"""

    conf = Configuration()

    if reset:
        confirm = click.confirm("Are you sure you want to rest configuration?")
        if confirm:
            conf.write(DEFAULT_CONFIG)
            click.echo("Configuration reset to default.")
        sys.exit(0)

    cfg = conf.read()
    clients = ["auto"] + CONTAINER_ENGINES
    cfg["engine"] = click.prompt(
        "Container Engine", type=click.Choice(clients), default=cfg["engine"]
    )
    cfg["image"] = click.prompt(
        "Selenium container image", default=cfg["image"]
    )
    viewers = ["auto"] + VNC_VIEWERS
    cfg["viewer"] = click.prompt(
        "VNC viewer", type=click.Choice(viewers), default=cfg["viewer"]
    )
    cfg["server_port"] = click.prompt(
        "Selenium server running on port?", default=cfg["server_port"]
    )
    cfg["vnc_port"] = click.prompt(
        "VNC running on port?", default=cfg["vnc_port"]
    )
    cfg["name"] = click.prompt("Selenium container name", default=cfg["name"])
    cfg["network"] = click.prompt(
        "Selenium container network", default=cfg["network"]
    )
    cfg["shm_size"] = click.prompt(
        "Selenium container shared Memory", default=cfg["shm_size"]
    )
    cfg["data_dir"] = click.prompt(
        "Testing data mount to directory '/data'", default=cfg["data_dir"]
    )

    conf.write(cfg=cfg)
    click.echo("Configuration saved successfully...")


_config = Configuration().read()



import shutil
import subprocess
import sys
import time

import click

from miqsel.config import _config
from miqsel.config import CONTAINER_ENGINES
from miqsel.config import VNC_VIEWERS


class SeleniumContainer:
    """Selenium Server Management"""

    def __init__(
        self,
        image=None,
        name=None,
        server_port=None,
        vnc_port=None,
        data_dir=None,
        network=None,
        shm_size=None,
        viewer=None,
    ):
        self.cfg = _config
        self.image = image or self.cfg["image"]
        self.name = name or self.cfg["name"]
        self.server_port = server_port or self.cfg["server_port"]
        self.vnc_port = vnc_port or self.cfg["vnc_port"]
        self.data_dir = data_dir or self.cfg["data_dir"]
        self.network = network or self.cfg["network"]
        self.shm_size = shm_size or self.cfg["shm_size"]
        self.viewer = viewer or self.cfg["viewer"]

    @property
    def client(self):
        _client = self.cfg["engine"]

        if _client == "auto":
            try:
                _client = next(c for c in CONTAINER_ENGINES if shutil.which(c))
            except StopIteration:
                click.echo(
                    f"No container engine found on machine. "
                    f"Install one of {CONTAINER_ENGINES}"
                )
        if shutil.which(_client):
            return _client
        else:
            click.echo(f"Container client {_client} not found")

    @property
    def is_running(self):
        cmd = subprocess.run([self.client, "ps"], stdout=subprocess.PIPE)
        return self.name in cmd.stdout.decode()

    @property
    def is_stale(self):
        cmd = subprocess.run([self.client, "ps", "-a"], stdout=subprocess.PIPE)
        return self.name in cmd.stdout.decode() and not self.is_running

    @property
    def executor(self):
        """
        Get executor url
        :return: If container running return executor url else None
        """

        return f"http://localhost:{self.server_port}/wd/hub"

    @property
    def vnc(self):
        """
        Get vnc url
        :return: If container running return vnc url else None
        """

        return f"localhost:{self.vnc_port}"

    def start(self, **kwargs):
        """Start selenium container"""
        if self.is_stale:
            cmd = [self.client, "rm", self.name]
            subprocess.run(cmd)

        if not self.is_running:
            cmd = [
                self.client,
                "run",
                "-d",
                "--rm",
                f"--shm-size={self.shm_size}",
                "--expose",
                str(self.vnc_port),
                "--expose",
                str(self.server_port),
                "-p",
                f"{self.vnc_port}:{self.vnc_port}",
                "-p",
                f"{self.server_port}:{self.server_port}",
                "-e",
                "SE_VNC_NO_PASSWORD=1",
                "--name",
                self.name,
            ]
            if self.network != "default":
                cmd.extend(["--network", self.network])
            if self.data_dir != "default":
                cmd.extend(["-v", f"{self.data_dir}:/data:z"])
            cmd.append(self.image)
            subprocess.run(cmd)
            # hack some time to stable
            time.sleep(2)

    def stop(self):
        """Stop selenium container if running"""
        subprocess.run([self.client, "stop", self.name])

    @property
    def status(self):
        """
        Get status of selenium container
        :return: container status if running else stopped
        """
        return "running" if self.is_running else "stopped"


@click.command(help="Status of Selenium Server")
def status():
    """status command"""

    miq = SeleniumContainer()
    click.echo(miq.status)


@click.command(help="Start Selenium Server")
@click.option("--viewer/--no-viewer", default=True)
@click.pass_context
def start(ctx, viewer):
    """start command"""

    miq = SeleniumContainer()
    if not miq.is_running:
        try:
            miq.start()
            click.echo("Selenium Server started")
            if viewer:
                ctx.invoke(viewer_cmd)

        except Exception:
            click.echo("Fail to start Selenium Server")
            raise
        # ctx.invoke(viewer)
    else:
        click.echo("Server in running state")


@click.command(help="Stop Selenium Server")
def stop():
    """stop command"""

    miq = SeleniumContainer()
    if miq.is_running:
        miq.stop()
        click.echo("Selenium Server stopped")
    else:
        click.echo("Server not running...")


@click.command(help="Selenium executor URL")
def executor():
    """executor url command"""

    miq = SeleniumContainer()
    if miq.status == "running":
        click.echo(miq.executor)
    else:
        click.echo("Server not running...")


@click.command(help="VNC URL")
def vnc():
    """vnc url command"""

    miq = SeleniumContainer()
    if miq.status == "running":
        click.echo(miq.vnc)
    else:
        click.echo("Server not running...")


@click.command("viewer", help="VNC viewer")
@click.option(
    "-u", "--url", default=None, help="Server url with port <hostname:port>"
)
def viewer_cmd(url):
    """vnc viewer command

    :param url: Server url with port <hostname:port>
    """
    miq = SeleniumContainer()

    if not miq.is_running:
        click.echo("Server not running...")
        sys.exit(1)

    url = url if url else miq.vnc

    try:
        _viewer = next(v for v in VNC_VIEWERS if shutil.which(v))
    except StopIteration:
        click.echo(f"No vnc viewer found. Install one of {VNC_VIEWERS}")

    subprocess.Popen([_viewer, url], stdout=subprocess.PIPE)



import os
import shutil
from subprocess import PIPE
from subprocess import Popen

import pytest

from tests import miqsel_cmd


@pytest.fixture(scope="module")
def config():
    """Configured miqsel"""
    if not os.path.isdir("conf"):
        os.mkdir("conf")
    proc = Popen(["miqsel", "config"], stdin=PIPE, stdout=PIPE)
    input = str.encode(f"{os.getcwd()}\n\n\n\n\n")
    proc.communicate(input=input)
    yield
    if os.path.isdir("conf"):
        shutil.rmtree("conf")


@pytest.fixture(scope="module")
def proj_dir():
    """Create Project dir"""

    # `conf` is identity that you are in project where we create evm.local.yaml
    if not os.path.isdir("conf"):
        os.mkdir("conf")
    yield "conf"
    if os.path.isdir("conf"):
        shutil.rmtree("conf")


@pytest.fixture(scope="function")
def ensure_stopped():
    out, _, _ = miqsel_cmd("miqsel status")
    if out.strip() == "running":
        miqsel_cmd("miqsel stop")


@pytest.fixture(scope="function")
def ensure_running():
    out, _, _ = miqsel_cmd("miqsel status")
    if out.strip() != "running":
        miqsel_cmd("miqsel start --no-viewer")



from tests import miqsel_cmd


def test_help():
    """Check Status of miqsel"""

    out, _, returncode = miqsel_cmd("miqsel")
    assert returncode == 0
    assert "Usage: miqsel [OPTIONS] COMMAND [ARGS]" in out


def test_config():
    pass


def test_executor(ensure_running):
    out, _, returncode = miqsel_cmd("miqsel executor")
    assert returncode == 0
    assert "http://localhost:4444/wd/hub" in out


def test_vnc(ensure_running):
    out, _, returncode = miqsel_cmd("miqsel vnc")
    assert returncode == 0
    assert "localhost:5999" in out


def test_start(ensure_stopped):
    out, _, returncode = miqsel_cmd("miqsel start --no-viewer")
    assert returncode == 0
    assert "Selenium Server started" in out


def test_status(ensure_stopped):
    out, _, returncode = miqsel_cmd("miqsel status")
    assert returncode == 0
    assert "stopped" in out
    miqsel_cmd("miqsel start --no-viewer")
    out, _, returncode = miqsel_cmd("miqsel status")
    assert returncode == 0
    assert "running" in out


def test_stop(ensure_running):
    out, _, returncode = miqsel_cmd("miqsel stop")
    assert returncode == 0
    assert "Selenium Server stopped" in out


def test_viewer():
    pass



import click

from miqsel.config import config
from miqsel.server import executor
from miqsel.server import start
from miqsel.server import status
from miqsel.server import stop
from miqsel.server import viewer_cmd
from miqsel.server import vnc


@click.version_option()
@click.group()
def main():
    """Miq Selenium Server"""
    pass


# config command
main.add_command(config)

# server management command
main.add_command(status)
main.add_command(start)
main.add_command(stop)
main.add_command(viewer_cmd)
main.add_command(vnc)
main.add_command(executor)


if __name__ == "__main__":
    main()
